"""
Model with helper functions for making api calls
"""
import requests
import json
import click

version = '3'


class ApiClient:
    """
    Client for making calls to the zotero api

    Args:
        api_key (str): a user generated api string
        user_id (str): the zotero user id to use as part of the base url

    Attributes:
        api_key (str): see api key arg
        base (str): base url string used for http requests
    """

    def __init__(self, api_key: str, user_id: str):
        self.api_key = api_key
        self.base = f'https://api.zotero.org/users/{user_id}/'

    def get_page(self, index: int, path: str) -> []:
        """
        Gets a single page of API response

        Args:
            index (int): The starting index for results. 
            path (str): The api path to make a request to

        Returns:
            list: JSON parsed text attribute of the response
        """
        return json.loads(
            requests.get(
                self.base + path,
                headers={
                    "Zotero-API-Version": version,
                    "Authorization": f"Bearer   {self.api_key}"
                },
                params={
                    'limit': '100',
                    'start': index
                }).text)

    def get_all_pages(self, path: str = ''):
        """
        Gets all available pages from zotero

        Args:
            path (str): The zotero api path to make a request to

        Returns:
            list: The JSON parsed text attribute of all pages of the reponse.
        """
        click.echo(click.style('Loading zotero web data...', fg='red'))
        index = 0
        final = []
        res = self.get_page(index, path)
        while len(res) != 0:
            final += res
            index += 100
            res = self.get_page(index, path)
        click.echo(click.style('Data Loaded.', fg='green'))
        return final



import click
from dataclasses import dataclass
from zotero_sync.api import ApiClient
import os
from pathlib import Path
from dotenv import load_dotenv, find_dotenv
load_dotenv(find_dotenv(filename='.zoterosync'))


@click.group()
def cli():
    """A command line tool for cleaning up zotfile directory.

    Paramaters can be passed as arguments or in a .zoterosync file.
    """
    pass


@dataclass
class pathsObj:
    computer_paths: list
    cloud_paths: list


def get_paths(file_dir: Path, client: ApiClient) -> pathsObj:
    """Returns both computer and cloud paths

    Args:
        file_dir (Path): location of zotfile directory
        client (ApiClient)

    Returns:
        pathsObj: A paths object
    """
    r = client.get_all_pages('items')
    cloud_paths = [path["data"]["path"]
                   for path in r if "path" in path["data"]]
    computer_paths = [path for path in file_dir.glob(
        "**/*.pdf") if "trash" not in str(path)]
    return pathsObj(cloud_paths=cloud_paths, computer_paths=computer_paths)


@cli.command()
@click.option('--file_dir',
              default=os.getenv('ZOTFILE_DIR'),
              type=click.Path(exists=True))
@click.option('--api_key',
              default=os.getenv('API_KEY'))
@click.option('--user_id',
              default=os.getenv('USER_ID'))
def trash(file_dir: click.Path, api_key: str, user_id: str):
    """Trash all files existing on system but not on cloud

    Args:
        file_dir (click.Path): location of zotfile directory
        api_key (str): zotero api key
        user_id (str): zotero user id
    """
    dotfile_explanation = (
        "This can be provided as an option or "
        "in a .zoterozync file as ZOTFILE_DIR")
    assert(file_dir is not None), (
        "No zotfile path was given. " + dotfile_explanation)
    assert(api_key is not None), (
        "No api key was given. " + dotfile_explanation)
    assert(user_id is not None), (
        "No user id was given. " + dotfile_explanation)
    client = ApiClient(api_key, user_id)
    file_dir = Path(file_dir)
    paths = get_paths(file_dir, client)
    computer_unique = [path for path in paths.computer_paths if str(
        path.absolute()) not in paths.cloud_paths]
    if click.confirm(
         f"Are you sure you want to trash {len(computer_unique)} variables?"):
        for path in computer_unique:
            trash = file_dir / 'trash'
            trash.mkdir(parents=True, exist_ok=True)
            path.rename(trash / path.name)


if __name__ == "__main__":
    cli()
